@@ -20,6 +20,7 @@
 #include "qgsgeometry.h"
 #include "qgsvectorlayerfeatureiterator.h"
 #include "qgssymbollayerreference.h"
+#include "qgsfeaturesink.h"
 
 class QgsFeatureRenderer;
 class QgsSymbolLayer;
@@ -204,11 +205,14 @@ class CORE_EXPORT QgsVectorLayerUtils
      * Duplicates a feature and it's children (one level deep). It calls CreateFeature, so
      * default values and constraints (e.g., unique constraints) will automatically be handled.
      * The duplicated feature will be automatically inserted into the layer.
-     * \a depth the higher this number the deeper the level - With depth > 0 the children of the feature are not duplicated
      * \a duplicateFeatureContext stores all the layers and the featureids of the duplicated features (incl. children)
+     * \a maxDepth the maximum depth to duplicate children in relations, 0 is unlimited depth (in any case, limited to 100)
+     * \a depth the current depth, not exposed in Python
+     * \a referencedLayersBranch the current branch of layers across the relations, not exposed in Python, taken by copy not reference, used to avoid infinite loop
      * \since QGIS 3.0
      */
-    static QgsFeature duplicateFeature( QgsVectorLayer *layer, const QgsFeature &feature, QgsProject *project, int depth, QgsDuplicateFeatureContext &duplicateFeatureContext SIP_OUT );
+    static QgsFeature duplicateFeature( QgsVectorLayer *layer, const QgsFeature &feature, QgsProject *project, QgsDuplicateFeatureContext &duplicateFeatureContext SIP_OUT, const int maxDepth = 0, int depth SIP_PYARGREMOVE = 0, QList<QgsVectorLayer *> referencedLayersBranch SIP_PYARGREMOVE = QList<QgsVectorLayer *>() );
+
 
     /**
      * Gets the feature source from a QgsVectorLayer pointer.
@@ -255,17 +259,20 @@ class CORE_EXPORT QgsVectorLayerUtils
      * converting a multi part geometry to single part
      *
      * The following operations will be performed to convert the input features:
-     *  - convert single geometries to multi part
-     *  - drop additional attributes
-     *  - drop geometry if layer is geometry-less
-     *  - add missing attribute fields
-     *  - add back M/Z values (initialized to 0)
-     *  - drop Z/M
-     *  - convert multi part geometries to single part
+     *
+     * - convert single geometries to multi part
+     * - drop additional attributes
+     * - drop geometry if layer is geometry-less
+     * - add missing attribute fields
+     * - add back M/Z values (initialized to 0)
+     * - drop Z/M
+     * - convert multi part geometries to single part
+     *
+     * Optionally, \a sinkFlags can be specified to further refine the compatibility logic.
      *
      * \since QGIS 3.4
      */
-    static QgsFeatureList makeFeatureCompatible( const QgsFeature &feature, const QgsVectorLayer *layer );
+    static QgsFeatureList makeFeatureCompatible( const QgsFeature &feature, const QgsVectorLayer *layer, QgsFeatureSink::SinkFlags sinkFlags = QgsFeatureSink::SinkFlags() );
 
     /**
      * Converts input \a features to be compatible with the given \a layer.
@@ -275,21 +282,24 @@ class CORE_EXPORT QgsVectorLayerUtils
      * of input features.
      *
      * The following operations will be performed to convert the input features:
-     *  - convert single geometries to multi part
-     *  - drop additional attributes
-     *  - drop geometry if layer is geometry-less
-     *  - add missing attribute fields
-     *  - add back M/Z values (initialized to 0)
-     *  - drop Z/M
-     *  - convert multi part geometries to single part
+     *
+     * - convert single geometries to multi part
+     * - drop additional attributes
+     * - drop geometry if layer is geometry-less
+     * - add missing attribute fields
+     * - add back M/Z values (initialized to 0)
+     * - drop Z/M
+     * - convert multi part geometries to single part
+     *
+     * Optionally, \a sinkFlags can be specified to further refine the compatibility logic.
      *
      * \since QGIS 3.4
      */
-    static QgsFeatureList makeFeaturesCompatible( const QgsFeatureList &features, const QgsVectorLayer *layer );
+    static QgsFeatureList makeFeaturesCompatible( const QgsFeatureList &features, const QgsVectorLayer *layer, QgsFeatureSink::SinkFlags sinkFlags = QgsFeatureSink::SinkFlags() );
 
     /**
-     * \return true if the \param feature field at index \param fieldIndex from \param layer
-     * is editable, false if the field is readonly
+     * \return TRUE if the \param feature field at index \param fieldIndex from \param layer
+     * is editable, FALSE if the field is readonly
      *
      * \since QGIS 3.10
      */
@@ -319,6 +329,26 @@ class CORE_EXPORT QgsVectorLayerUtils
      * \since QGIS 3.12
      */
     static QString getFeatureDisplayString( const QgsVectorLayer *layer, const QgsFeature &feature );
+
+    /**
+     * Flags that can be used when determining cascaded features.
+     *
+     * \since QGIS 3.4
+     */
+    enum CascadedFeatureFlag
+    {
+      IgnoreAuxiliaryLayers = 1 << 1, //!< Ignore auxiliary layers
+    };
+    Q_DECLARE_FLAGS( CascadedFeatureFlags, CascadedFeatureFlag )
+
+    /**
+     * \returns TRUE if at least one feature of the \a fids on \a layer is connected as parent in at
+     * least one composition relation of the \a project or contains joins, where cascade delete is set.
+     * Details about cascading effects will be written to \a context.
+     * \since QGIS 3.14
+     */
+    static bool impactsCascadeFeatures( const QgsVectorLayer *layer, const QgsFeatureIds &fids, const QgsProject *project, QgsDuplicateFeatureContext &context SIP_OUT, QgsVectorLayerUtils::CascadedFeatureFlags flags = QgsVectorLayerUtils::CascadedFeatureFlags() );
+
 };
 
 
